@@ -12,6 +12,10 @@ from typing import Any, Iterable, Tuple
 
 import numpy as np
 
+from .utils import (format_for_output, format_str_case,
+                    generate_name_description_error_message,
+                    SUPPORTED_STR_FORMAT_OPTIONS)
+
 
 __all__ = ["processed_property"]
 
@@ -32,6 +36,20 @@ def processed_property(name, **kwargs):
         The metaprogrammed property object with setter including specified
         settings.
     """
+
+    def parse_kwarg(kwarg_name, description, valids, default):
+        kwarg = kwargs.get(kwarg_name, default)
+        if isinstance(valids, type) and not isinstance(kwarg, valids):
+            msg = (f"{repr(kwarg)} is not a valid {description}. Please "
+                   f"use a value of type {repr(valids)}.")
+            raise TypeError(msg)
+        elif not isinstance(valids, type) and kwarg not in valids:
+            formatted_valids = format_for_output(valids, with_or=True)
+            msg = (f"{repr(kwarg)} is not a valid {description}. Please "
+                   "choose one of: {formatted_valids}.")
+            raise ValueError(msg)
+        return kwarg
+
     storage_name = "_" + name
     description = kwargs.get("description")
     expected_type = kwargs.get("type")
@@ -47,6 +65,13 @@ def processed_property(name, **kwargs):
     exclusive = kwargs.get("exclusive", False)
     optimisable = kwargs.get("optimisable", False)
     post_method = kwargs.get("method")
+    less_than = kwargs.get("less_than")
+    greater_than = kwargs.get("greater_than")
+    at_least = kwargs.get("at_least")
+    at_most = kwargs.get("at_most")
+    equal_to = kwargs.get("equal_to")
+    str_format = parse_kwarg("str_format", "string case format",
+                             SUPPORTED_STR_FORMAT_OPTIONS, None)
 
     @property
     def prop(self):
@@ -87,13 +112,27 @@ def processed_property(name, **kwargs):
             check_min(value)
         if max_value is not None:
             check_max(value)
+        if less_than is not None:
+            check_less_than(self, value)
+        if greater_than is not None:
+            check_greater_than(self, value)
+        if at_least is not None:
+            check_at_least(self, value)
+        if at_most is not None:
+            check_at_most(self, value)
+        if equal_to is not None:
+            check_equal_to(self, value)
         if len_sequence is not None:
             check_len(value, len_sequence)
         if optimisable:
             value = process_optimisable(value)
+        if str_format:
+            value = format_str_case(value, str_format)
         if post_method is not None:
             value = apply_method(value)
         setattr(self, storage_name, value)
+        setattr(self, f"{storage_name}_dir", {"name": name,
+                                              "description": description})
 
     def check_type(value):
         """Ensure the type of the property value to be set is as specified.
@@ -128,7 +167,8 @@ def processed_property(name, **kwargs):
         elif cast_to_type:
             return cast_type(value)
         else:
-            name_str = generate_name_description_error_message()
+            name_str = generate_name_description_error_message(name,
+                                                               description)
             msg = (f"{name_str} must be a {repr(expected_type)}, instead got "
                    f"a {repr(type(value))}.")
             raise TypeError(msg)
@@ -158,7 +198,8 @@ def processed_property(name, **kwargs):
         try:
             exec(cast_str)
         except (ValueError, TypeError) as e:
-            name_str = generate_name_description_error_message()
+            name_str = generate_name_description_error_message(name,
+                                                               description)
             msg = (f"{name_str} must be a {repr(expected_type)}, instead got "
                    f"a {repr(type(value))} which cannot be cast.")
             raise e(msg)
@@ -192,6 +233,8 @@ def processed_property(name, **kwargs):
             formatted_unsupported_option = format_for_output(value,
                                                              with_verb=True)
             formatted_description = generate_name_description_error_message(
+                name,
+                description,
                 with_preposition=True)
             msg = (
                 f"{formatted_unsupported_option} not currently supported as "
@@ -201,7 +244,8 @@ def processed_property(name, **kwargs):
             raise ValueError(msg)
         elif value not in options:
             formatted_value = format_for_output(value, with_verb=True)
-            name_str = generate_name_description_error_message()
+            name_str = generate_name_description_error_message(name,
+                                                               description)
             msg = (
                 f"{formatted_value} not a valid option of {name_str}. "
                 f"Choose one of: {formatted_valid_options}.")
@@ -222,7 +266,10 @@ def processed_property(name, **kwargs):
             If the value attempting to be set is less than the specified
             minimum.
         """
-        name_str = generate_name_description_error_message()
+        name_str = generate_name_description_error_message(
+            name,
+            description,
+            is_sentence_start=True)
         if exclusive:
             if value <= min_value:
                 msg = (f"{name_str} must be greater than {min_value}. {value} "
@@ -249,7 +296,10 @@ def processed_property(name, **kwargs):
             If the value attempting to be set is less than the specified
             maximum.
         """
-        name_str = generate_name_description_error_message()
+        name_str = generate_name_description_error_message(
+            name,
+            description,
+            is_sentence_start=True)
         if exclusive:
             if value >= max_value:
                 msg = (f"{name_str} must be less than {max_value}. {value} is "
@@ -261,59 +311,57 @@ def processed_property(name, **kwargs):
                        f"{max_value}. {value} is invalid.")
                 raise ValueError(msg)
 
-    def generate_name_description_error_message(is_sentence_start=False,
-                                                with_preposition=False):
-        """Combine the name and description for correctly-formatted error.
+    def check_less_than(self, value):
+        def less_than_lambda(val_1, val_2):
+            return val_1 < val_2
+        check_comparison(self, value, less_than, "less than",
+                         less_than_lambda)
 
-        Parameters
-        ----------
-        is_sentence_start : bool, optional
-            Should the formatted str be return in title case.
-        with_preposition : bool, optional
-            Should 'a' or 'an' be preappended to the error message.
+    def check_greater_than(self, value):
+        def greater_than_lambda(val_1, val_2):
+            return val_1 > val_2
+        check_comparison(self, value, greater_than, "greater than",
+                         greater_than_lambda)
 
-        Returns
-        -------
-        str
-            Formatted description.
-        """
-        if description is None:
-            return f"`{name}`"
-        if with_preposition:
-            first_word, _ = description.split(maxsplit=1)
-            starts_with_vowel = first_word[0] in {"a", "e", "h", "i", "o", "u"}
-            is_acronym = first_word.upper() == first_word
-            if starts_with_vowel or is_acronym:
-                preposition = "an"
-            else:
-                preposition = "a"
-            formatted_description = " ".join([preposition, description])
+    def check_at_least(self, value):
+        def at_least_lambda(val_1, val_2):
+            return val_1 >= val_2
+        check_comparison(self, value, at_least, "at least",
+                         at_least_lambda)
+
+    def check_at_most(self, value):
+        def at_most_lambda(val_1, val_2):
+            return val_1 <= val_2
+        check_comparison(self, value, at_most, "at most",
+                         at_most_lambda)
+
+    def check_equal_to(self, value):
+        def equal_to_lambda(val_1, val_2):
+            return val_1 == val_2
+        check_comparison(self, value, equal_to, "equal to",
+                         equal_to_lambda)
+
+    def check_comparison(self, value, other, comparison_description,
+                         comparison_func):
+        try:
+            other_value = getattr(self, other)
+        except AttributeError:
+            pass
         else:
-            formatted_description = description
-        if is_sentence_start:
-            formatted_description = make_title_case(formatted_description)
-        return f"{formatted_description} (`{name}`)"
-
-    def make_title_case(description):
-        """Returns a str in title case.
-
-        Correctly formats title case handling scenario when appreviations are
-        included in the name/description.
-
-        Parameters
-        ----------
-        description : str
-            Description for formatting
-
-        Returns
-        -------
-        str
-            Formatted description
-        """
-        if len(description) > 1:
-            title_description = description[0].upper() + description[1:]
-            return title_description
-        return description.upper()
+            if not comparison_func(value, other_value):
+                other_dir = getattr(self, f"_{other}_dir")
+                other_name = other_dir["name"]
+                other_description = other_dir["description"]
+                name_str = generate_name_description_error_message(
+                    name, description, is_sentence_start=True)
+                other_name_str = generate_name_description_error_message(
+                    other_name, other_description)
+                value_formatted = format_for_output(value)
+                other_value_formatted = format_for_output(other_value)
+                msg = (f"{name_str} with value {value_formatted} must be "
+                       f"{comparison_description} {other_name_str} with "
+                       f"value {other_value_formatted}.")
+                raise ValueError(msg)
 
     def check_len(value, len_sequence):
         """Enforces the set sequence length to be equal to a specified value.
@@ -332,7 +380,8 @@ def processed_property(name, **kwargs):
 
         """
         if len(value) != len_sequence:
-            name_str = generate_name_description_error_message()
+            name_str = generate_name_description_error_message(name,
+                                                               description)
             msg = (f"{name_str} must be a sequence of length {len_sequence}.")
             raise ValueError(msg)
 
@@ -377,7 +426,7 @@ def processed_property(name, **kwargs):
             the single supplied value is not of type `numbers.Real`.
 
         """
-        name_str = generate_name_description_error_message()
+        name_str = generate_name_description_error_message(name, description)
         if isinstance(value, Real):
             return value
         if isinstance(value, Iterable):
@@ -439,118 +488,3 @@ def processed_property(name, **kwargs):
         return tuple(bounds)
 
     return prop
-
-
-def format_case(item, case):
-    """Allow :obj:`str` case formatting method application from keyword.
-
-    Parameters
-    ----------
-    item : str
-        Item to be case formatted.
-    case : str
-        Which case format method to use.
-
-    Returns
-    -------
-    str
-        :arg:`item` with case method applied.
-    """
-    if case == "title":
-        return item.title()
-    elif case == "upper":
-        return item.upper()
-    elif case == "lower":
-        return item.lower()
-    else:
-        return item
-
-
-def format_for_output(items, *args, **kwargs):
-    """Utility method for formatting console output.
-
-    Passes directly to :func:`format_multiple_items_for_output` just with a
-    shorter function name.
-
-    Parameters
-    ----------
-    items : iterable
-        Items to be formatted for output.
-    *args
-        Variable length argument list.
-    **kwargs
-        Arbitrary keyword arguments.
-
-    Returns
-    -------
-    str
-        Formatted string for console output.
-    """
-    return format_multiple_items_for_output(items, *args, **kwargs)
-
-
-def format_multiple_items_for_output(items, wrapping_char="'", *,
-                                     prefix_char="", case=None,
-                                     with_verb=False, with_or=False):
-    """Format multiple items for pretty console output.
-
-    Parameters
-    ----------
-    items : iterable of str
-        Items to be formatted.
-    wrapping_char : str (default `"'"`)
-        Prefix and suffix character for format wrapping.
-    prefix_char : str (default `""`)
-        Additional prefix.
-    case : str (default `None`)
-        Keyword for :func:`format_case`.
-    with_verb : bool, optional (default `False`)
-        Append the correct conjugation of "is"/"are" to end of list.
-    with_or : bool, optional
-        Description
-
-    Returns
-    -------
-    str
-        Formatted string of multiple items for console output.
-    """
-    items = format_as_iterable(items)
-    items = [f"{prefix_char}{format_case(item, case)}" for item in items]
-    if len(items) == 1:
-        formatted_items = f"{wrapping_char}{items[0]}{wrapping_char}"
-    else:
-        pad = f"{wrapping_char}, {wrapping_char}"
-        joiner = "or" if with_or else "and"
-        formatted_items = (f"{wrapping_char}{pad.join(items[:-1])}"
-                           f"{wrapping_char} {joiner} {wrapping_char}"
-                           f"{items[-1]}{wrapping_char}")
-    verb = "is" if len(items) == 1 else "are"
-    if with_verb:
-        formatted_items = f"{formatted_items} {verb}"
-
-    return formatted_items
-
-
-def format_as_iterable(items):
-    """Checks whether an item can be iterated over like a list.
-
-    Parameters
-    ----------
-    items : Any
-        Object to be tested for whether it is a list-like iterable.
-
-    Returns
-    -------
-    Union[Tuple[Any], Iterable[Any]]
-        If the passed object is already a list-like iterable then this is
-        returned straight away. Otherwise, the object is inserted in to a tuple
-        of length 1.
-
-    """
-    if isinstance(items, str):
-        return (items, )
-    try:
-        iter(items)
-    except TypeError:
-        return (items, )
-    return items
